/*
 * $Id: base_pkgs.h,v 1.1 1999/07/28 15:13:39 casper Exp $
 *
 * List of Solaris 8 base pacakges; these are the packages that have
 * proper file permissions in Solaris 8 and they should not end up in the
 * contents.mods file.
 *
 * The package names in the file must be sorted alphabetically.
 */
	"SUNWaccr",
	"SUNWaccu",
	"SUNWami",
	"SUNWamid",
	"SUNWamidx",
	"SUNWamir",
	"SUNWamix",
	"SUNWapppr",
	"SUNWapppu",
	"SUNWarc",
	"SUNWarcx",
	"SUNWast",
	"SUNWatfsr",
	"SUNWatfsu",
	"SUNWauda",
	"SUNWaudd",
	"SUNWauddx",
	"SUNWaudh",
	"SUNWbcp",
	"SUNWbnur",
	"SUNWbnuu",
	"SUNWbtool",
	"SUNWbtoox",
	"SUNWcar",
	"SUNWcarx",
	"SUNWcg6",
	"SUNWcg6h",
	"SUNWcg6x",
	"SUNWcpr",
	"SUNWcprx",
	"SUNWcry",
	"SUNWcry64",
	"SUNWcryr",
	"SUNWcryrx",
	"SUNWcsd",
	"SUNWcsl",
	"SUNWcslx",
	"SUNWcsr",
	"SUNWcstl",
	"SUNWcstlx",
	"SUNWcsu",
	"SUNWcsxu",
	"SUNWcvc",
	"SUNWcvcr",
	"SUNWcvcx",
	"SUNWdfb",
	"SUNWdfbh",
	"SUNWdhcm",
	"SUNWdhcsr",
	"SUNWdhcsu",
	"SUNWdoc",
	"SUNWdpl",
	"SUNWdplx",
	"SUNWdrr",
	"SUNWdrrx",
	"SUNWesu",
	"SUNWesxu",
	"SUNWfac",
	"SUNWfns",
	"SUNWfnsx",
	"SUNWfnsx5",
	"SUNWfnx5x",
	"SUNWftpr",
	"SUNWftpu",
	"SUNWgsdhx",
	"SUNWgss",
	"SUNWgssc",
	"SUNWgssdh",
	"SUNWgssk",
	"SUNWgsskx",
	"SUNWgssx",
	"SUNWhea",
	"SUNWhmd",
	"SUNWhmdu",
	"SUNWhmdx",
	"SUNWidn",
	"SUNWidnx",
	"SUNWifp",
	"SUNWifph",
	"SUNWifpx",
	"SUNWipc",
	"SUNWipcx",
	"SUNWkey",
	"SUNWkvm",
	"SUNWkvmx",
	"SUNWllc",
	"SUNWllcr",
	"SUNWllcx",
	"SUNWlldap",
	"SUNWloc",
	"SUNWlocx",
	"SUNWluxd",
	"SUNWluxdx",
	"SUNWluxl",
	"SUNWluxlx",
	"SUNWluxop",
	"SUNWmibii",
	"SUNWnisr",
	"SUNWnisu",
	"SUNWntpr",
	"SUNWntpu",
	"SUNWosdem",
	"SUNWpcelx",
	"SUNWpcmci",
	"SUNWpcmcu",
	"SUNWpcmcx",
	"SUNWpcmem",
	"SUNWpcr",
	"SUNWpcser",
	"SUNWpcu",
	"SUNWpd",
	"SUNWpdu",
	"SUNWpdx",
	"SUNWpl5p",
	"SUNWpl5u",
	"SUNWpmr",
	"SUNWpmu",
	"SUNWpmux",
	"SUNWppm",
	"SUNWpppk",
	"SUNWpppkx",
	"SUNWpsdpr",
	"SUNWpsf",
	"SUNWpsr",
	"SUNWpstl",
	"SUNWpstlx",
	"SUNWpsu",
	"SUNWrsg",
	"SUNWrsgk",
	"SUNWrsgx",
	"SUNWsacom",
	"SUNWsadmi",
	"SUNWsadmx",
	"SUNWsasnm",
	"SUNWsasnx",
	"SUNWscplp",
	"SUNWscpr",
	"SUNWscpu",
	"SUNWscpux",
	"SUNWses",
	"SUNWsesx",
	"SUNWslpr",
	"SUNWslpu",
	"SUNWslpx",
	"SUNWsndmr",
	"SUNWsndmu",
	"SUNWsra",
	"SUNWsrh",
	"SUNWssad",
	"SUNWssadx",
	"SUNWssaop",
	"SUNWstatl",
	"SUNWsutl",
	"SUNWsvvs",
	"SUNWsvvsx",
	"SUNWsxr",
	"SUNWter",
	"SUNWtnfc",
	"SUNWtnfcx",
	"SUNWtnfd",
	"SUNWtoo",
	"SUNWtoox",
	"SUNWudf",
	"SUNWudfr",
	"SUNWudfrx",
	"SUNWusb",
	"SUNWusbu",
	"SUNWusbx",
	"SUNWuxfl1",
	"SUNWuxfl2",
	"SUNWuxfl4",
	"SUNWuxfle",
	"SUNWuxflr",
	"SUNWuxflu",
	"SUNWvolr",
	"SUNWvolu",
	"SUNWvolux",
	"SUNWvygdr",
	"SUNWxcu4",
	"SUNWxwdv",
	"SUNWxwdvx",
	"SUNWypr",
	"SUNWypu",
